@@ -2,20 +2,21 @@
 Custom Authenticator to use Azure AD with JupyterHub
 """
 
-import json
-import jwt
 import os
 import urllib
+from distutils.version import LooseVersion as V
 
-from tornado.auth import OAuth2Mixin
-from tornado.log import app_log
-from tornado.httpclient import HTTPRequest, AsyncHTTPClient
-
+import jwt
 from jupyterhub.auth import LocalAuthenticator
-
+from tornado.httpclient import HTTPRequest
 from traitlets import Unicode, default
 
-from .oauth2 import OAuthLoginHandler, OAuthenticator
+from .oauth2 import OAuthenticator
+
+
+# pyjwt 2.0 has changed its signature,
+# but mwoauth pins to pyjwt 1.x
+PYJWT_2 = V(jwt.__version__) >= V("2.0")
 
 
 class AzureAdOAuthenticator(OAuthenticator):
@@ -47,7 +48,6 @@ class AzureAdOAuthenticator(OAuthenticator):
 
     async def authenticate(self, handler, data=None):
         code = handler.get_argument("code")
-        http_client = AsyncHTTPClient()
 
         params = dict(
             client_id=self.client_id,
@@ -72,14 +72,20 @@ class AzureAdOAuthenticator(OAuthenticator):
             body=data  # Body is required for a POST...
         )
 
-        resp = await http_client.fetch(req)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+        resp_json = await self.fetch(req)
 
-        # app_log.info("Response %s", resp_json)
         access_token = resp_json['access_token']
-
         id_token = resp_json['id_token']
-        decoded = jwt.decode(id_token, options={"verify_signature": False})
+
+        if PYJWT_2:
+            decoded = jwt.decode(
+                id_token,
+                options={"verify_signature": False},
+                audience=self.client_id,
+            )
+        else:
+            # pyjwt 1.x
+            decoded = jwt.decode(id_token, verify=False)
 
         userdict = {"name": decoded[self.username_claim]}
         userdict["auth_state"] = auth_state = {}
