@@ -2,20 +2,14 @@
 Custom Authenticator to use Bitbucket OAuth with JupyterHub
 """
 
-import json
 import urllib
 
-from tornado.auth import OAuth2Mixin
-from tornado import web
-
-from tornado.httputil import url_concat
-from tornado.httpclient import HTTPRequest, AsyncHTTPClient
-
 from jupyterhub.auth import LocalAuthenticator
+from tornado.httpclient import HTTPRequest
+from tornado.httputil import url_concat
+from traitlets import Set, default
 
-from traitlets import Set, default, observe
-
-from .oauth2 import OAuthLoginHandler, OAuthenticator
+from .oauth2 import OAuthenticator
 
 
 def _api_headers(access_token):
@@ -60,8 +54,6 @@ class BitbucketOAuthenticator(OAuthenticator):
 
     async def authenticate(self, handler, data=None):
         code = handler.get_argument("code")
-        # TODO: Configure the curl_httpclient for tornado
-        http_client = AsyncHTTPClient()
 
         params = dict(
             client_id=self.client_id,
@@ -83,8 +75,7 @@ class BitbucketOAuthenticator(OAuthenticator):
             headers=bb_header,
         )
 
-        resp = await http_client.fetch(req)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+        resp_json = await self.fetch(req)
 
         access_token = resp_json['access_token']
 
@@ -94,8 +85,7 @@ class BitbucketOAuthenticator(OAuthenticator):
             method="GET",
             headers=_api_headers(access_token),
         )
-        resp = await http_client.fetch(req)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+        resp_json = await self.fetch(req)
 
         username = resp_json["username"]
 
@@ -113,7 +103,6 @@ class BitbucketOAuthenticator(OAuthenticator):
         }
 
     async def _check_membership_allowed_teams(self, username, access_token):
-        http_client = AsyncHTTPClient()
 
         headers = _api_headers(access_token)
         # We verify the team membership by calling teams endpoint.
@@ -122,8 +111,7 @@ class BitbucketOAuthenticator(OAuthenticator):
         )
         while next_page:
             req = HTTPRequest(next_page, method="GET", headers=headers)
-            resp = await http_client.fetch(req)
-            resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+            resp_json = await self.fetch(req)
             next_page = resp_json.get('next', None)
 
             user_teams = set([entry["username"] for entry in resp_json["values"]])
