@@ -5,23 +5,16 @@ Custom Authenticator to use GitLab OAuth with JupyterHub
 
 import json
 import os
-import re
-import sys
 import warnings
 from urllib.parse import quote
 
-from tornado.auth import OAuth2Mixin
-from tornado import web
-
-from tornado.escape import url_escape
-from tornado.httputil import url_concat
-from tornado.httpclient import HTTPRequest, AsyncHTTPClient
-
 from jupyterhub.auth import LocalAuthenticator
+from tornado.escape import url_escape
+from tornado.httpclient import HTTPRequest
+from tornado.httputil import url_concat
+from traitlets import CUnicode, Set, Unicode, default
 
-from traitlets import Set, CUnicode, Unicode, default, observe
-
-from .oauth2 import OAuthLoginHandler, OAuthenticator
+from .oauth2 import OAuthenticator
 
 
 def _api_headers(access_token):
@@ -116,8 +109,6 @@ class GitLabOAuthenticator(OAuthenticator):
 
     async def authenticate(self, handler, data=None):
         code = handler.get_argument("code")
-        # TODO: Configure the curl_httpclient for tornado
-        http_client = AsyncHTTPClient()
 
         # Exchange the OAuth code for a GitLab Access Token
         #
@@ -144,9 +135,7 @@ class GitLabOAuthenticator(OAuthenticator):
             body='',  # Body is required for a POST...
         )
 
-        resp = await http_client.fetch(req)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
-
+        resp_json = await self.fetch(req, label="getting access token")
         access_token = resp_json['access_token']
 
         # memoize gitlab version for class lifetime
@@ -161,8 +150,7 @@ class GitLabOAuthenticator(OAuthenticator):
             validate_cert=validate_server_cert,
             headers=_api_headers(access_token),
         )
-        resp = await http_client.fetch(req)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+        resp_json = await self.fetch(req, label="getting gitlab user")
 
         username = resp_json["username"]
         user_id = resp_json["id"]
@@ -207,14 +195,12 @@ class GitLabOAuthenticator(OAuthenticator):
             headers=_api_headers(access_token),
             validate_cert=self.validate_server_cert,
         )
-        resp = await AsyncHTTPClient().fetch(req, raise_error=True)
-        resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+        resp_json = await self.fetch(req)
         version_strings = resp_json['version'].split('-')[0].split('.')[:3]
         version_ints = list(map(int, version_strings))
         return version_ints
 
     async def _check_membership_allowed_groups(self, user_id, access_token):
-        http_client = AsyncHTTPClient()
         headers = _api_headers(access_token)
         # Check if user is a member of any group in the allowed list
         for group in map(url_escape, self.allowed_gitlab_groups):
@@ -225,13 +211,12 @@ class GitLabOAuthenticator(OAuthenticator):
                 user_id,
             )
             req = HTTPRequest(url, method="GET", headers=headers)
-            resp = await http_client.fetch(req, raise_error=False)
+            resp = await self.fetch(req, raise_error=False, parse_json=False)
             if resp.code == 200:
                 return True  # user _is_ in group
         return False
 
     async def _check_membership_allowed_project_ids(self, user_id, access_token):
-        http_client = AsyncHTTPClient()
         headers = _api_headers(access_token)
         # Check if user has developer access to any project in the allowed list
         for project in self.allowed_project_ids:
@@ -242,15 +227,13 @@ class GitLabOAuthenticator(OAuthenticator):
                 user_id,
             )
             req = HTTPRequest(url, method="GET", headers=headers)
-            resp = await http_client.fetch(req, raise_error=False)
-
-            if resp.body:
-                resp_json = json.loads(resp.body.decode('utf8', 'replace'))
+            resp_json = await self.fetch(req, raise_error=False)
+            if resp_json:
                 access_level = resp_json.get('access_level', 0)
 
                 # We only allow access level Developer and above
                 # Reference: https://docs.gitlab.com/ee/api/members.html
-                if resp.code == 200 and access_level >= 30:
+                if access_level >= 30:
                     return True
         return False
 
